import 'dart:io';

import 'package:circumstellar/circumstellar.dart';

import 'package:w_transport/vm.dart' show vmTransportPlatform;

void main() {
  var logger = new LoggerContext.root('Circumstellar Test Bot');
  logger.log(LogLevel.Info, 'Starting up..');
  var client = new DiscordClient(
      new DiscordAuthSet(
          Platform.environment['CIRCUMSTELLAR_TEST_TOKEN'], TokenType.Bot),
      vmTransportPlatform);
  client.dispatcher.messageCreate.listen((Message msg) async {
    if (msg.content == '!channel') {
      await msg.channel.createMessage('', embed: {
        'title': 'Channel info for: ${msg.channel.name}',
        'description': 'Discord channel description',
        'fields': [
          {
            'name': 'NSFW',
            'value': msg.channel.isNsfw.toString(),
            'inline': true
          },
          {'name': 'Topic', 'value': msg.channel.topic, 'inline': true},
          {'name': 'Guild ID', 'value': msg.channel.guildId, 'inline': true}
        ]
      });
    }
    if (msg.content.startsWith('!greet')) {
      await msg.channel.createMessage('Hello ${msg.mentions[0].username}!');
    }
    if (msg.content == '!guild') {
      await msg.channel.createMessage('', embed: {
        'title': 'Server info for: ${msg.channel.guild.name}',
        'description': 'Discord guild (server) description',
        'fields': [
          {'name': 'Region', 'value': msg.channel.guild.region, 'inline': true},
          {
            'name': 'Owner Username',
            'value': msg.channel.guild.owner.username,
            'inline': true
          },
          {
            'name': 'Icon URL',
            'value': msg.channel.guild.getIconUrl('png'),
            'inline': true
          },
          {
            'name': 'AFK Timeout (Seconds)',
            'value': msg.channel.guild.afkTimeoutSeconds,
            'inline': true
          },
          {
            'name': 'Verification Level',
            'value': GuildVerificationLevelHelper
                .getLevelFromGuild(msg.channel.guild),
            'inline': true
          }
        ]
      });
    }
  });
  client.dispatcher.ready.listen((empty) {
    logger.log(LogLevel.Info, 'Ready');
  });
  client.dispatcher.channelCreate.listen((chan) async {
    if (chan.type == ChannelType.GuildText) {
      var chan0 = (chan as GuildTextChannel);
      logger.log(LogLevel.Info, 'Channel ${chan0.name} has been created!');
    }
  });
  client.dispatcher.channelUpdate.listen((chan) async {
    if (chan.type == ChannelType.GuildText) {
      var chan1 = (chan as GuildTextChannel);
      logger.log(LogLevel.Info, 'Channel ${chan1.name} has been updated!');
    }
  });
  client.dispatcher.guildUpdate.listen((guild) async {
    logger.log(LogLevel.Info, 'Guild ${guild.name} has been updated!');
  });
  client.start();
}
