import 'dart:async';

import 'package:circumstellar/circumstellar.dart';

class EntityManager {
  DiscordClient _client;
  Map<String, Entity> _cache = {};

  EntityManager(this._client);

  void deleteEntity(String id) {
    _cache.remove(id);
  }

  void updateEntity(String id, Entity newEntity, String typeName) {
    _cache[id] = newEntity;
  }

  Future<Entity> getEntity(String id, String typeName,
      [Map<String, dynamic> newData]) async {
    Entity finalObject = null;

    var cacheResult = _cache[id];
    if (cacheResult != null) {
      finalObject = cacheResult;
      return finalObject;
    }

    if (typeName == 'Guild') {
      finalObject = new Guild(_client);
      if (newData != null) {
        await finalObject.updateModel(newData, _client);
      } else {
        await finalObject.updateModel(
            (await _client.rest.executeRoute(
                    ApiRoutes.guilds.getGuildWithId(id.toString())))
                .json,
            _client);
      }
    }

    if (typeName == 'GuildTextChannel') {
      finalObject = new GuildTextChannel(_client);
      if (newData != null) {
        await finalObject.updateModel(newData, _client);
      } else {
        await finalObject.updateModel(
            (await _client.rest.executeRoute(
                    ApiRoutes.channels.getChannelWithId(id.toString())))
                .json,
            _client);
      }
    }

    if (typeName == 'Message') {
      finalObject = new Message(_client);
      if (newData != null) {
        await finalObject.updateModel(newData, _client);
      } else
        throw new CacheException('Cannot create Message object from cache');
    }

    if (typeName == 'User') {
      finalObject = new User(_client);
      if (newData != null) {
        await finalObject.updateModel(newData, _client);
      } else {
        await finalObject.updateModel(
            (await _client.rest
                    .executeRoute(ApiRoutes.users.getUserWithId(id.toString())))
                .json,
            _client);
      }
    }

    if (finalObject != null) this.updateEntity(id, finalObject, typeName);
    return _cache[id];
  }
}
